import os, sys, string, re

def is_appdir(path):
	"Returns true if path is a valid AppDir."
	apprun_path = os.path.realpath(path + os.sep + "AppRun")
	x = os.access(apprun_path, os.X_OK)
	return x

def find_appdir(path, search_string, case_sensitive=0):
	"""Searches the path for an AppDir and returns the full path to it.
Returns the null string if the AppDir does not exist."""
	app_dir = ''
	if case_sensitive:
		search_str = string.lower(search_string)
	else:
		search_str = search_string
	search_dirs = string.split(path, ":")
	for dir in search_dirs:
		try:
			files = os.listdir(dir)
		except:
			# Use the empty sequence to make the for loop not 
			files = []
		for entry in files:
			if case_sensitive:
				f = entry
			else:
				f = string.lower(entry)
			if f == search_str:
				full_path = os.path.normpath(dir + os.sep + entry)
				# If this is an AppDir, then we're done.
				if is_appdir(full_path):
					app_dir = os.path.realpath(full_path)
					return app_dir
	return app_dir

def find_platform():
	"Returns the current platform as used in AppRun scripts."
	outs, errs = os.popen2('uname -m')
	arch = ''
	try:	
		arch = arch + outs.readline()
		arch = string.strip(arch)
	except:
		pass
	try:
		arch = arch + errs.readline()
		arch = string.strip(arch)
	except:
		pass
	try:
		outs.close()
		errs.close()
	except:
		pass
	outs, errs = os.popen2('uname -s')
	opsys = ''
	try:
		opsys = opsys + outs.readline()
		opsys = string.strip(opsys)
	except:
		pass
	try:
		opsys = opsys + errs.readline()
		opsys = string.strip(opsys)
	except:
		pass
	try:
		outs.close()
		errs.close()
	except:
		pass
	if re.match("i.86", arch):
		arch = "ix86"
	ret = opsys + "-" + arch
	return ret

def list_appdirs(path):
	"Returns a list of all AppDirs in directory path."
	ret = []
	try:
		files = os.listdir(path)
	except:
		files = []
	for f in files:
		if is_appdir(f): ret.append(f)
	return ret

def find_readable_choices():
	"Reads the CHOICESPATH and returns the first readable directory."
	path = os.getenv("CHOICESPATH")
	if path == None: 
		path = os.getenv("HOME") + "/Choices:/usr/share/Choices:/usr/local/share/Choices"
	search_dirs = string.split(path, ":")
	for d in search_dirs:
		if os.access(d, os.R_OK):
			return d
	return ''

def find_writable_choices():
	"Reads the CHOICESPATH and returns the first writable directory."
	path = os.getenv("CHOICESPATH")
	if path == None: 
		path = os.getenv("HOME") + "/Choices:/usr/share/Choices:/usr/local/share/Choices"
	search_dirs = string.split(path, ":")
	for d in search_dirs:
		if os.access(d, os.W_OK):
			return d
	return ''

def find_readable_app_choices(appname, case_sensitive=0):
	"Finds a the first readable Choices directory for an application."
	path = os.getenv("CHOICESPATH")
	if case_sensitive:
		search_str = appname
	else:
		search_str = string.lower(appname)
	if path == None: 
		path = os.getenv("HOME") + "/Choices:/usr/share/Choices:/usr/local/share/Choices"
	choices_dirs = string.split(path, ":")
	for cd in choices_dirs:
		try:
			files = os.listdir(cd)
		except:
			files = []
		for entry in files:
			full_path = os.path.realpath(cd + os.sep + entry)
			if not os.path.isdir(full_path): continue
			if case_sensitive:
				f = entry
			else:
				f = string.lower(entry)
			if search_str == f and os.access(full_path, os.R_OK):
				return full_path
	return ''

def find_writable_app_choices(appname, case_sensitive=0, create=0):
	"Finds a the first writeable Choices directory for an application."
	path = os.getenv("CHOICESPATH")
	if case_sensitive:
		search_str = appname
	else:
		search_str = string.lower(appname)
	if path == None: 
		path = os.getenv("HOME") + "/Choices:/usr/share/Choices:/usr/local/share/Choices"
	choices_dirs = string.split(path, ":")
	for cd in choices_dirs:
		try:
			files = os.listdir(cd)
		except:
			files = []
		for entry in files:
			full_path = os.path.realpath(cd + os.sep + entry)
			if not os.path.isdir(full_path): continue
			if case_sensitive:
				f = entry
			else:
				f = string.lower(entry)
			if search_str == f and os.access(full_path, os.W_OK):
				return full_path
	if not create: return ''
	ret = find_writable_choices()
	full_path = os.path.normpath(ret + os.sep + appname)
	os.makedir(full_path)
	return full_path

def get_binary_names(appdir_path):
	"""Takes the path to an AppDir and returns a list of executables 
in the directory for the current platform."""
	if not is_appdir(appdir_path): return []
	bindir = appdir_path + find_platform()
	ret = []
	try:
		files = os.listdir(bindir)
	except:
		files = []
	for f in files:
		full_path = os.path.normpath(bindir + os.sep + f)
		if os.path.isfile(full_path) and os.access(full_path, os.X_OK):
			ret.append(f)
	return ret
